import re
import os
import sys
import sqlite3
from . import CNYEARDB_PATH

class Cnyeardb(object):
    def __init__(self, dbpath=CNYEARDB_PATH,debug=False):
        if debug:
            self.debugger = self.debugger_t
        else:
            self.debugger = self.debugger_f
        self.sqlbase = '''SELECT 
            tb_regnal_postq.id,
            tb_dynasty_c1_postq.name,
            tb_dynasty_c1_postq.alias,
            tb_dynasty_c2_postq.name,
            tb_dynasty_c2_postq.alias,
            tb_dynasty_postq.name,
            tb_dynasty_postq.alias,
            tb_emperor_postq.name,
            tb_emperor_postq.temple_name,
            tb_emperor_postq.alias,
            tb_regnal_postq.regnal,
            tb_regnal_postq.alias,
            tb_regnal_postq.ya,
            tb_regnal_postq.yz
            FROM tb_regnal_postq
            LEFT JOIN tb_dynasty_postq ON tb_regnal_postq.dynasty_id = tb_dynasty_postq.id
            LEFT JOIN tb_emperor_postq ON tb_regnal_postq.emperor_id = tb_emperor_postq.id
            LEFT JOIN tb_dynasty_c1_postq ON tb_dynasty_postq.c1_id = tb_dynasty_c1_postq.id
            LEFT JOIN tb_dynasty_c2_postq ON tb_dynasty_postq.c2_id = tb_dynasty_c2_postq.id
            '''
        def regexp(expr, item):
            reg = re.compile(expr)
            return reg.search(item) is not None
        try:
            self.conn = sqlite3.connect(dbpath)
            self.c = self.conn.cursor()
            self.conn.create_function("REGEXP",2,regexp)
        except Error as e:
            print(e)
            sys.exit(-1)
        self.regp_dnst = self.mkregp_dnst()
        self.regp_rgnl = self.mkregp_rgnl()
        self.recpl_dnst = re.compile(self.regp_dnst)
        self.recpl_rgnl = re.compile(self.regp_rgnl)

    def debugger_t(self, foo):
        print(foo)
    def debugger_f(self, foo):
        pass

    def setdebug(self):
        self.debugger = self.debugger_t
    def unsetdebug(self):
        self.debugger = self.debugger_f

    def lkp_year(self, year):
        sqlcmd = self.sqlbase + 'WHERE (tb_regnal_postq.ya <= %s) AND (tb_regnal_postq.yz >= %s)' %(year, year)
        self.c.execute(sqlcmd)
        return self.c.fetchall()

    def mksqlcmd_dnst(self,d,da='',d1='',d1a='',d2='',d2a=''):
        if not d: return None
        sqld = ''
        sqld1 = ''
        sqld2 = ''
        if d or da:
            if d:
                sqld = "tb_dynasty_postq.name='%s'" %d
            if da:
                sqld += " OR " if d else ""
                sqld += "tb_dynasty_postq.alias REGEXP '%s'" %da
        if sqld: sqld = "(%s)" %sqld
        if d1 or d1a:
            if d1:
                sqld1 = "tb_dynasty_c1_postq.name='%s'" %d1
            if d1a:
                sqld1 += " OR " if d1 else ""
                sqld1 += "tb_dynasty_c1_postq.alias REGEXP '%s'" %d1a
        if sqld1: sqld1 = "(%s)" %sqld1
        if d2 or d2a:
            if d2:
                sqld2 = "tb_dynasty_c2_postq.name='%s'" %d2
            if d2a:
                sqld2 += " OR " if d2 else ""
                sqld2 += "tb_dynasty_c2_postq.alias REGEXP '%s'" %d2a
        if sqld2: sqld2 = "(%s)" %sqld2
        # sqld must not be none
        sqlcmd = sqld
        if sqld1: sqlcmd = "%s AND %s" %(sqlcmd,sqld1)
        if sqld2: sqlcmd = "%s AND %s" %(sqlcmd,sqld2)
        self.debugger(sqlcmd)
        return sqlcmd

    def mksqlcmd_rgnl(self, d, rgnl, rgnla='', da='', d1='', d1a='', d2='', d2a=''):
        if not rgnl: return None
        sqlcmd_dnst = self.mksqlcmd_dnst(d=d,da=da,d1=d1,d1a=d1a,d2=d2,d2a=d2a)
        s = "tb_regnal_postq.regnal='%s'" %rgnl
        s += " OR tb_dynasty_c2_postq.alias REGEXP '%s'" %rgnla if rgnla else ""
        s = "(%s)" %s
        sqlcmd = sqlcmd_dnst + ' AND ' + s
        self.debugger(sqlcmd)
        return sqlcmd

    def lkp_rgnl(self, d, rgnl, rgnla='', da='', d1='', d1a='', d2='', d2a=''):
        sqlcmd = self.mksqlcmd_rgnl(d=d,rgnl=rgnl,rgnla=rgnla,da=da,d1=d1,d1a=d1a,d2=d2,d2a=d2a)
        sqlcmd = self.sqlbase + "WHERE " + sqlcmd
        self.c.execute(sqlcmd)
        return self.c.fetchall()

    def lkp_dnst(self, d, da='', d1='', d1a='', d2='', d2a=''):
        sqlcmd = self.mksqlcmd_dnst(d=d,da=da,d1=d1,d1a=d1a,d2=d2,d2a=d2a)
        sqlcmd = self.sqlbase + "WHERE " + sqlcmd
        self.c.execute(sqlcmd)
        return self.c.fetchall()

    def mkregp_dnst(self):
        try:
            if self.regp_dnst: return self.regp_dnst # AttributeError then do the following
        except:
            pass
        self.c.execute('''SELECT name, alias FROM tb_dynasty_postq''')
        sqlrd = self.c.fetchall()
        self.c.execute('''SELECT name, alias FROM tb_dynasty_c1_postq''')
        sqlrd1 = self.c.fetchall()
        self.c.execute('''SELECT name, alias FROM tb_dynasty_c2_postq''')
        sqlrd2 = self.c.fetchall()
        exprd = []
        exprd1 = []
        exprd2 = []
        for t in sqlrd:
            exprd.append(t[0])
            exprd += t[1].split(' ') if t[1] else []
        for t in sqlrd1:
            exprd1.append(t[0])
            exprd1 += t[1].split(' ') if t[1] else []
        for t in sqlrd2:
            exprd2.append(t[0])
            exprd2 += t[1].split(' ') if t[1] else []
        expr = '(%s)?(%s)?(%s)' %('|'.join(exprd1),'|'.join(exprd2),'|'.join(exprd))
        self.debugger(expr)
        return expr

    def mtch_dnst(self, s):
        m = self.recpl_dnst.search(s)
        if not m: return None
        return [m.group(1),m.group(2),m.group(3)]

    def mkregp_rgnl(self):
        try:
            if self.regp_rgnl: return self.regp_rgnl # AttributeError then do the following
        except:
            pass
        self.c.execute('''SELECT name, alias FROM tb_dynasty_postq''')
        sqlrd = self.c.fetchall()
        self.c.execute('''SELECT name, alias FROM tb_dynasty_c1_postq''')
        sqlrd1 = self.c.fetchall()
        self.c.execute('''SELECT name, alias FROM tb_dynasty_c2_postq''')
        sqlrd2 = self.c.fetchall()
        self.c.execute('''SELECT regnal, alias FROM tb_regnal_postq''')
        sqlrr = self.c.fetchall()
        exprd = []
        exprd1 = []
        exprd2 = []
        exprr = []
        for t in sqlrd:
            exprd.append(t[0])
            exprd += t[1].split(' ') if t[1] else []
        for t in sqlrd1:
            exprd1.append(t[0])
            exprd1 += t[1].split(' ') if t[1] else []
        for t in sqlrd2:
            exprd2.append(t[0])
            exprd2 += t[1].split(' ') if t[1] else []
        for t in sqlrr:
            if t[0]:
                exprr.append(t[0])
            exprr += t[1].split(' ') if t[1] else []
        expr = '(%s)?(%s)?(%s)(%s)' %('|'.join(exprd1),'|'.join(exprd2),'|'.join(exprd),'|'.join(exprr))
        expr += '([元一二三四五六七八九十]+)年'
        self.debugger(expr)
        return expr

    def mtch_rgnl(self, s):
        m = self.recpl_rgnl.search(s)
        if not m: return None
        return [m.group(1),m.group(2),m.group(3),m.group(4),m.group(5)]




'''
--------------------- 
作者：mattkang 
来源：CSDN 
原文：https://blog.csdn.net/handsomekang/article/details/52563487?utm_source=copy 
版权声明：本文为博主原创文章，转载请附上博文链接！
'''
import re

def convert(n):
    '''accept int (not float) and convert it to chinese digit numbers (str)'''
    #units = ['', '万', '亿']
    #nums = ['零', '壹', '贰', '叁', '肆', '伍', '陆', '柒', '捌', '玖']
    #decimal_label = ['角', '分']
    #small_int_label = ['', '拾', '佰', '仟']
    #
    units = ['', '万', '亿']
    nums = ['零', '一', '二', '三', '四', '五', '六', '七', '八', '九']
    #decimal_label = ['角', '分']
    small_int_label = ['', '十', '百', '千']
    #
    #int_part, decimal_part = str(int(n)), str(n - int(n))[2:]  # 分离整数和小数部分
    #
    int_part = str(int(n)) # only accept int, no float
    #

    res = []
    #if decimal_part:
    #    res.append(''.join([nums[int(x)] + y for x, y in zip(decimal_part, decimal_label) if x != '0']))

    if int_part == '0':
        return '〇'
    if int_part != '0':
        #res.append('圆')
        while int_part:
            small_int_part, int_part = int_part[-4:], int_part[:-4]
            tmp = ''.join([nums[int(x)] + (y if x != '0' else '') for x, y in list(zip(small_int_part[::-1], small_int_label))[::-1]])
            tmp = tmp.rstrip('零').replace('零零零', '零').replace('零零', '零')
            unit = units.pop(0)
            if tmp:
                tmp += unit
                res.append(tmp)
    #return ''.join(res[::-1])
    #
    # Concerning 一十五 -> 十五
    r = ''.join(res[::-1])
    if r[:2] == '一十':
        r = r[1:]
    return r
    #
dig2cn = convert

def year_dig2cn(s):
   '''convert 清乾隆12年 清亁隆十二年
             清乾隆1年 清乾隆元年
             清乾隆元年 清乾隆元年
             1898年 一八九八年
             abcde abcde
   '''
   c_cn = re.compile(r'([0-9]+)年')
   m = c_cn.search(s)
   if not m: return s
   s_dig = m.group(1)
   dig = int(s_dig)
   if len(s_dig) == 4: # CE
       r = ''.join(dig2cn(int(x)) for x in s_dig)
   else:
       if dig == 1:
           r = '元'
       else:
           r = dig2cn(dig)
   return s.replace(s_dig+'年',r+'年')


if __name__ == '__main__':
    print(convert(1000))
    print(dig2cn(10))
    print(dig2cn(15))
    print(dig2cn(23))
    for i in range(0, 100):
        print(dig2cn(i))
    print(year_dig2cn('清乾隆元年'))
    print(year_dig2cn('清乾隆1年'))
    print(year_dig2cn('清乾隆2年'))
    print(year_dig2cn('民國2年'))
    print(year_dig2cn('民國間'))
    print(year_dig2cn('1898年'))
    print(year_dig2cn('1890年'))





import os
import sys
import pkg_resources
#pkg_dir = os.path.dirname(os.path.abspath(__file__))

CNYEARDB_PATH = pkg_resources.resource_filename('cnyeardb','cnyeardb.db')


from .cnyeardb_obj import Cnyeardb
from .cnyear_obj import Cnyear




# Convert chinese numbers to arabic numbers
# copied from http://www.iplaypy.com/code/base/b2600.html
# and modified into a py3 version
# Not chinese number: return None
import re

CN_NUM = {
    u'〇': 0,
    u'一': 1,
    u'二': 2,
    u'三': 3,
    u'四': 4,
    u'五': 5,
    u'六': 6,
    u'七': 7,
    u'八': 8,
    u'九': 9,

    u'零': 0,
    u'壹': 1,
    u'贰': 2,
    u'叁': 3,
    u'肆': 4,
    u'伍': 5,
    u'陆': 6,
    u'柒': 7,
    u'捌': 8,
    u'玖': 9,

    u'貮': 2,
    u'两': 2,
}
CN_UNIT = {
    u'十': 10,
    u'拾': 10,
    u'百': 100,
    u'佰': 100,
    u'千': 1000,
    u'仟': 1000,
    u'万': 10000,
    u'萬': 10000,
    u'亿': 100000000,
    u'億': 100000000,
    u'兆': 1000000000000,
}


def cn2dig(cn):
    lcn = list(cn)
    unit = 0  # 当前的单位
    ldig = []  # 临时数组

    while lcn:
        cndig = lcn.pop()

        if cndig in CN_UNIT:
            unit = CN_UNIT.get(cndig)
            if unit == 10000:
                ldig.append('w')  # 标示万位
                unit = 1
            elif unit == 100000000:
                ldig.append('y')  # 标示亿位
                unit = 1
            elif unit == 1000000000000:  # 标示兆位
                ldig.append('z')
                unit = 1

            continue

        else:
            dig = CN_NUM.get(cndig)

            if unit:
                dig = dig * unit
                unit = 0

            ldig.append(dig)

    if unit == 10:  # 处理10-19的数字
        ldig.append(10)

    ret = 0
    tmp = 0

    while ldig:
        x = ldig.pop()

        if x == 'w':
            tmp *= 10000
            ret += tmp
            tmp = 0

        elif x == 'y':
            tmp *= 100000000
            ret += tmp
            tmp = 0

        elif x == 'z':
            tmp *= 1000000000000
            ret += tmp
            tmp = 0

        else:
            if tmp==None or x==None:# not a chinese number
                return None
            tmp += x

    ret += tmp
    return ret

    # ldig.reverse()
    # print ldig
    # print CN_NUM[u'七']
    """
    def cn2dig_all(s):
        '''convert all cn to dig in a string'''
        c_cn = re.compile('([零一二三四五六七八九十百千万亿兆]+)')
        j = c_cn.finditer(s)
        lpos = [] # to save all the positions to be converted later
        ldig = [] # to save all the converted digits
        for m in j:
            c_cn = 
    """
def year_cn2dig(s, conv_1=False):
    '''convert 清乾隆十二年 清乾隆12年
              一八九〇年 1890年
              一八九零年 1890年
              清乾隆元年 清乾隆元年
    conv_1: whether to convert 元年 to 1年
    '''
    if conv_1:
        if '元年' in s:
            return s.replace('元年','1年')
    c_cn = re.compile(r'([一二三四五六七八九十〇零]+)年')
    m = c_cn.search(s)
    if not m: return s
    s_dig = m.group(1)
    if len(s_dig) == 4: # CE
        r = ''.join(str(cn2dig(x)) for x in s_dig)
    else:
        r = cn2dig(s_dig)
    return s.replace(s_dig+'年', str(r)+'年')
    
if __name__ == '__main__':

    test_dig = [u'九',
                u'十一',
                u'一百二十三',
                u'一千二百零三',
                u'一万一千一百零一',
                u'十万零三千六百零九',
                u'一百二十三万四千五百六十七',
                u'一千一百二十三万四千五百六十七',
                u'一亿一千一百二十三万四千五百六十七',
                u'一百零二亿五千零一万零一千零三十八',
                u'一千一百一十一亿一千一百二十三万四千五百六十七',
                u'一兆一千一百一十一亿一千一百二十三万四千五百六十七',
                u'空'
                ]

    for cn in test_dig:
        print (cn2dig(cn))
    print(year_cn2dig('一八九零年'))
    print(year_cn2dig('一八九〇年'))
    print(year_cn2dig('乾隆元年',True))
    print(year_cn2dig('乾隆元年'))
    print(year_cn2dig('乾隆十二年'))
    print(year_cn2dig('乾隆六十年'))
    print(year_cn2dig('民國二十八年'))
    print(year_cn2dig('清咸豐間'))
    print(cn2dig('清'))



from setuptools import setup, find_packages

setup(
        name='cnyear',
        version = '0.0.5',
        description = 'A tool for conversion between Chinese year and CE.',
        author = 'Nemo',
        author_email = 'nonellf@gmail.com',
        url = 'https://github.com/nemo-nullius/cnyear',
        license = 'GPLv3',
        classifiers = [
            'Development Status :: 3 - Alpha',
            'Intended Audience :: Developers',
            'Intended Audience :: Education',
            'Intended Audience :: Science/Research',
            'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
            'Topic :: Database',
            'Programming Language :: Python :: 3.7',
            ],
        keywords = 'Chinese regnal year CE conversion',
        packages = find_packages(exclude=['tests']),
        package_data = {
            # If package cnyeardb contains *.db, include it
            'cnyeardb':['*.db']},
        )

